#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include "game.h"
#include "piece.h"
#include "utils.h"




struct game_s{
  piece *tabpieces;
  int nb_moves;
  int nb_pieces;
  int width;
  int height;
};

//*
static bool cross_pieces (game g,int piece_num, dir d , int distance);
static bool compatible_direction(cpiece p,dir d);
static bool pieces_in_board(game g ,cpiece p, dir d,int distance);
//*/

game new_game_hr (int nb_pieces, piece *pieces){
  return new_game(6,6,nb_pieces,pieces);

}


void delete_game (game g){
  for (int i=0;i<g->nb_pieces;i++){
    delete_piece(g->tabpieces[i]);
  }
  free (g->tabpieces);
  free (g);
}



void copy_game (cgame src, game dst){
  dst->nb_moves=src->nb_moves;
  piece *temp;
  for (int i = 0 ; i < dst->nb_pieces; i++)
   	delete_piece(dst->tabpieces[i]);
  temp = realloc(dst->tabpieces,(src->nb_pieces)*sizeof(piece));
  if (temp == NULL){
    fprintf(stderr,"probleme de Reallocation\n");
    exit(EXIT_FAILURE);
  }
  dst->nb_pieces=src->nb_pieces;
  dst->tabpieces=temp;
  dst->width=src->width;
  dst->height=src->height;

  for (int i=0 ;i<dst->nb_pieces;i++){
    dst->tabpieces[i]=new_piece(get_x(src->tabpieces[i]),get_y(src->tabpieces[i]),get_width(src->tabpieces[i]),get_height(src->tabpieces[i]),can_move_x(src->tabpieces[i]),can_move_y(src->tabpieces[i]));
  }
}





int game_nb_pieces(cgame g){
  return g->nb_pieces;
}




cpiece game_piece(cgame g, int piece_num){
  return g->tabpieces[piece_num];
}



bool game_over_hr(cgame g){
  return (get_x(g->tabpieces[0])==4 && get_y(g->tabpieces[0])==3);
}

bool play_move(game g, int piece_num, dir d, int distance){
  if (compatible_direction(g->tabpieces[piece_num],d) && pieces_in_board(g,g->tabpieces[piece_num],d,distance) && cross_pieces(g,piece_num,d,distance)){
    g->nb_moves+=1;
    return true;
  }
  return false;
}

int game_nb_moves(cgame g){
  return g->nb_moves;
}

static bool compatible_direction(cpiece p,dir d){
	if ((d==LEFT || d==RIGHT) && can_move_x(p))
		return true;
	else if ((d==UP || d==DOWN) && can_move_y(p))
		return true;
	else 
		return false;
}

static bool pieces_in_board(game g, cpiece p, dir d,int distance){	
	int x=get_x(p);
	int y=get_y(p);
  int width=get_width(p)-1;
  int height=get_height(p)-1;
	if (d==UP){
		return (y+height+distance<g->height);
	}
	else if (d==DOWN){
		return (y-distance>=0);
	}
	else if (d==RIGHT){
		return (x+width+distance<g->width);
	}
	else if (d==LEFT){
		return (x-distance>=0);
	}
	return false;
}

static bool cross_pieces (game g,int piece_num, dir d , int distance){
	dir inverse_moves=(d-2)%4;
	for (int i=0;i<distance;i++){
		move_piece(g->tabpieces[piece_num],d,1);
		for (int j=0 ; j< g->nb_pieces;j++){
			if (j==piece_num)
				continue;
			if (intersect(g->tabpieces[piece_num],g->tabpieces[j])){
				move_piece(g->tabpieces[piece_num],inverse_moves,i+1);
				return false;
			}
		}
	}
	return true;
}



/////////////VERSION 2////////////////////////////////////


game new_game (int width, int height, int nb_pieces, piece *pieces){
  game g = (game)malloc(sizeof(struct game_s));
  if (g == NULL){
    fprintf(stderr,"probleme d'allocation\n");
    exit(EXIT_FAILURE);
  }
  else{
    g->tabpieces =malloc((nb_pieces)*sizeof(piece)); //NB_pieces + Redcar
    if(g->tabpieces==NULL){
      fprintf(stderr,"probleme d'allocation\n");
      exit(EXIT_FAILURE);
    }
  }
  g->nb_moves=0;
  g->nb_pieces=nb_pieces;
	g->width=width;
	g->height=height;
  for (int i=0; i<g->nb_pieces;i++){
    g->tabpieces[i]=new_piece(get_x(pieces[i]),get_y(pieces[i]),get_width(pieces[i]),get_height(pieces[i]),can_move_x(pieces[i]),can_move_y(pieces[i]));
  }
	return g;
}

int game_width(cgame g){
	return g->width;
}

int game_height(cgame g){
	return g->height;
}

int game_square_piece (cgame g, int x, int y){
	int x1,y1,width,height;
	for (int i=0 ;i<g->nb_pieces;i++){
		x1=get_x(g->tabpieces[i]);
		y1=get_y(g->tabpieces[i]);
    width=get_width(g->tabpieces[i])-1;
    height=get_height(g->tabpieces[i])-1;
    if ((x1<=x) && (x<=(x1+width)) && (y1<=y) && (y<=(y1+height)))
      return i;
  }
  return -1;
}






