#include <stdlib.h>
#include <time.h>
#include <stdio.h>
#include "game.h"
#include "piece.h"
#include "utils.h"

#define NB_PIECES 5

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 * @return true if expected==value
 */
bool test_equality_int(int expected, int value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}

/**
 * @brief test if value is equal to expected; if not, displays an error message containing msg to standard error output 
 */
bool test_equality_bool(bool expected, bool value, char * msg) {
  if (expected != value)
    fprintf(stderr,"ERR: value expected %d ; value computed %d. %s\n", expected, value, msg);
  return expected == value;
}

piece pieces[NB_PIECES];


/* configue de test
.....3
.....3
...003
......
...122
...1..
 */


void set_up() {
  pieces[0] = new_piece_rh(0, 3, true, true);
  pieces[1] = new_piece_rh(3, 3, true, true);
  pieces[2] = new_piece_rh(3, 0, true, false);
  pieces[3] = new_piece_rh(4, 1, true, true);
  pieces[4] = new_piece_rh(5, 3, false, false);
}
void tear_down() {
  for (int i = 0 ; i < NB_PIECES; i++)
    delete_piece(pieces[i]);
}

bool compare_piece(cpiece p,cpiece g){
  bool result = true;
  result= result && test_equality_int(get_x(g), get_x(p),"x_compare");
  result= result && test_equality_int(get_y(g),get_y(p),"y_compare");
  result= result && test_equality_bool(is_horizontal(g),is_horizontal(p),"horizontal_compare");


  return result;
}

bool test_new_game(){
  bool result = true;
  set_up();
  game g = new_game_hr(NB_PIECES,pieces);
  result=result && test_equality_int(0,game_nb_moves(g),"game_nb_moves");
  result=result && test_equality_int(NB_PIECES,game_nb_pieces(g),"game_nb_pieces");
  result=result && test_equality_bool(false,game_over_hr(g), "game_over_hr");
  for (int i=0 ; i<NB_PIECES;i++){
    result=result && test_equality_bool(true,compare_piece(game_piece(g,i),pieces[i]),"compare_piece");
  }
  delete_game(g);
  tear_down();
  return result;
}

bool test_copy_game(){
  bool result = true;
  set_up();
  game g = new_game_hr(NB_PIECES,pieces);
  piece pieces2[1];
  pieces2[0]=new_piece_rh(0,0,true,true);
  game p = new_game_hr(0,pieces2);
  copy_game(g, p);
  result=result && test_equality_int(game_nb_moves(p),game_nb_moves(g),"game_nb_moves");
  result=result && test_equality_int(game_nb_pieces(p),game_nb_pieces(g),"game_nb_pieces");
  result=result && test_equality_bool(game_over_hr(p),game_over_hr(g),"game_over_hr");
  result=result && test_equality_bool(game_width(p),game_width(g),"game_width");
  result=result && test_equality_bool(game_height(p),game_height(g),"game_height");
  for (int i=0 ; i<NB_PIECES;i++){
    result=result && test_equality_bool(true,compare_piece(game_piece(g,i),game_piece(p,i)),"compare_piece");
  }
  tear_down();
  delete_piece(pieces2[0]);
  delete_game(g);
  delete_game(p);
  return result;
}


bool test_play_move(){
  bool result = true;
  set_up();
  game g = new_game_hr(NB_PIECES,pieces);
  result=result && test_equality_bool(true,play_move(g,0,RIGHT,1),"play_move_redcar_RIGHT_1");
  result=result && test_equality_bool(false,play_move(g,0,UP,1),"play_move_redcar_UP_1");
  result=result && test_equality_bool(false,play_move(g,0,RIGHT,2),"play_move_redcar_Intersect");
  result=result && test_equality_bool(false,play_move(g,4,UP,1),"play_move_4eme_UP_1_Hors_zone");
  delete_game(g);
  return result;
}


bool test_game_square(){
  bool result=true;
  set_up();
  game g = new_game_hr(NB_PIECES,pieces);
  result = result && test_equality_int(0,game_square_piece (g,0,3),"(0;3) square");
  result = result && test_equality_int(4,game_square_piece(g,5,3),"(5;4) square");
  delete_game(g);
  return result;
}

int main (int argc, char *argv[]){
  bool result= true;
  result = result && test_equality_bool(true, test_new_game(), "test_new_game");
  result = result && test_equality_bool(true, test_copy_game(), "test_copy_game");
  result = result && test_equality_bool(true, test_play_move(), "test_play_move");
  result = result && test_equality_bool(true, test_game_square(), "test_game_square");

  if (result) {
    printf("all test of test.game.c succeeded \n");
    return EXIT_SUCCESS;
  }
  else
    return EXIT_FAILURE;
}
